﻿#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup(){
    webcam.setup(1280, 720);

    contour.setMinAreaRadius(1);
    contour.setMaxAreaRadius(5);
    color_red_laser.set(185, 160, 160);
    calibrated = 0;
    key_pressed = 0;

}

//--------------------------------------------------------------
void ofApp::update(){
    webcam.update();

    if (webcam.isFrameNew()) {
        contour.setTargetColor(color);
        contour.setThreshold(30);
        contour.findContours(webcam);
        selection = 0;
        if (contour.size() != 0) {
            for (i = 1; i < contour.size(); i++) {
                if (contour.getContourArea(i) > contour.getContourArea(selection)) {
                    selection = i;
                }
            }
            if ((color_picked == 1) && (contour.getContourArea(selection) != 0)) {
                center_trapeze = contour.getCenter(selection);
            }
        }
        
        
    }
}

//--------------------------------------------------------------
void ofApp::draw(){
    ofSetColor(255);
    webcam.draw(0, 0);

    contour.draw();
    
    ofFill();
    ofSetColor(color);
    ofDrawRectangle(webcam.getWidth(), 0, 128, 128);
    ofDrawLine(cp_top_left.x, cp_top_left.y, cp_top_right.x, cp_top_right.y);
    ofDrawLine(cp_top_right.x, cp_top_right.y, cp_bottom_right.x, cp_bottom_right.y);
    ofDrawLine(cp_bottom_right.x, cp_bottom_right.y, cp_bottom_left.x, cp_bottom_left.y);
    ofDrawLine(cp_bottom_left.x, cp_bottom_left.y, cp_top_left.x, cp_top_left.y);
    ofDrawLine(middle, cp_top_left.y, middle, cp_bottom_right.y);      
    if (color_picked == 1 && key_pressed == 0) {
        scaleTrapeze();
        scaleRectangel();
        SetCursorPos(cp_cursor_position.x, cp_cursor_position.y);
    }
    
}

//--------------------------------------------------------------
void ofApp::keyPressed(int key){
    key_pressed = 1;
}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){
    key_pressed = 0;
}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button) {
    if (calibrated == 4) {
        color = webcam.getPixels().getColor(x, y);
        printf("%i, %i, %i\n", color.r, color.g, color.b);
        printf("%f, %f, %f, %f \n", y_offset, y_scale, x_offset, x_scale);
        color_picked = 1;
    }
    if (calibrated != 4 && key_pressed == 0) {
        if (calibrated == 0) {
            GetCursorPos(&cp_top_left);
        }
        if (calibrated == 1) {
            GetCursorPos(&cp_top_right);
        }
        if (calibrated == 2) {
            GetCursorPos(&cp_bottom_right);
        }
        if (calibrated == 3) {
            GetCursorPos(&cp_bottom_left);
            if (cp_top_left.x > cp_bottom_left.x)
            {
                x_offset = cp_bottom_left.x;
            }
            else
            {
                x_offset = cp_top_left.x;
            }

            if (cp_top_right.x < cp_bottom_right.x)
            {
                x_scale = ofGetScreenWidth() / (cp_bottom_right.x - x_offset);
            }
            else
            {
                x_scale = ofGetScreenWidth() / (cp_top_right.x - x_offset);
            }
            
            if (cp_top_left.y > cp_top_right.y)
            {
                y_offset = cp_top_right.y;
            }
            else
            {
                y_offset = cp_top_left.y;
            }
            
            if (cp_bottom_left.y < cp_bottom_right.y)
            {
                y_scale = ofGetScreenHeight() / (cp_bottom_right.y - y_offset);
            }
            else
            {
                y_scale = ofGetScreenHeight() / (cp_bottom_left.y - y_offset);
            }
           
            printf("Y:%f \nX:%f\n", y_offset, x_offset);
        }
        calibrated++;
    }
}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseEntered(int x, int y){

}

//--------------------------------------------------------------
void ofApp::mouseExited(int x, int y){

}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){ 

}


void ofApp::scaleTrapeze(void) {
    left_pitch = (float)(cp_bottom_left.y - cp_top_left.y) / (float)(cp_bottom_left.x - cp_top_left.x);
    right_pitch = (float) (cp_bottom_right.y - cp_top_right.y) / (float)(cp_bottom_right.x - cp_top_right.x);
    middle = (cp_top_right.x + cp_top_left.x)/2;
    stringstream out;
    out.precision(2);
    out << "Middle:" << middle; 
    string message = out.str();
    ofDrawBitmapString(message, 100, 100);
    out.str("");
    out << "rightpitch:" << right_pitch;
    message = out.str();
    ofDrawBitmapString(message, 100, 120);
    out.str("");
    out << "leftpitch:" << left_pitch;
    message = out.str();
    ofDrawBitmapString(message, 100, 140);
    if (center_trapeze.x > middle) {
        float mid_to_dot = center_trapeze.x - middle;
        float distance_to_rect = ((cp_bottom_right.y - center_trapeze.y) / right_pitch);
        float mid_to_max = (cp_bottom_right.x - middle) - distance_to_rect;
        center_rect.x = center_trapeze.x + ((distance_to_rect) * ((mid_to_dot)/(mid_to_max)));
    }
    else {
        float mid_to_dot = middle - center_trapeze.x;
        float distance_to_rect = ((cp_bottom_left.y - center_trapeze.y) / left_pitch);
        float mid_to_max = (middle - cp_bottom_left.x) - distance_to_rect;
        center_rect.x = center_trapeze.x + ((distance_to_rect) * ((mid_to_dot)/(mid_to_max)));
    }
}

void ofApp::scaleRectangel(void) {
    cp_cursor_position.x = (center_rect.x - x_offset) * x_scale;
    cp_cursor_position.y = (center_trapeze.y - y_offset )* y_scale ;
    stringstream out;
    out.precision(2);
    out << "Cursor:" << "x" << cp_cursor_position.x << " | y"<< cp_cursor_position.y;
    string message = out.str();
    ofDrawBitmapString(message, 100, 160);
    out.str("");
}